/*============================================================================
  File:     Safe Plans and Plan Guides.sql

  Summary:  This query helps you to determine how much of your cache is used
			by plans that have the same query hash but have not been parameterized. 
			This shows query_hash and query_plan_hash.
			
  Date:     October 2011

  SQL Server Version: 2005/2008
------------------------------------------------------------------------------
  Written by SQLskills

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/


-- For more code, content and scripts/demos - see the blog post category
-- Optimizing Procedural Code on Kimberly's blog:
-- http://www.sqlskills.com/BLOGS/KIMBERLY/category/Optimizing-Procedural-Code.aspx

--------------------------------------------------------
-- What if we can create a "SAFE" and consistent plan?
-- In this case, we might have single plan because of a
-- covering index...
--------------------------------------------------------

-- What if we have a query that can be covered... 
-- And, it's run frequently...

-- Run a few select statements:
USE Credit
go

SELECT m.lastname, m.Firstname, m.phone_no 
FROM dbo.member AS m
WHERE m.lastname = 'Tripp';
go

SELECT m.lastname, m.firstname, m.phone_no 
FROM dbo.member AS m
WHERE m.lastname = 'Jones';
go

SELECT m.lastname, m.firstname, m.phone_no 
FROM dbo.member AS m
WHERE m.lastname = 'Smith';
go

SELECT m.lastname, m.firstname, m.phone_no 
FROM dbo.member AS m
WHERE m.lastname = 'Anderson';
go

SELECT m.lastname, m.firstname, m.phone_no 
FROM dbo.member AS m
WHERE m.lastname = 'Test';
go

CREATE INDEX test 
ON member(lastname, firstname, phone_no)
go

-- If all of our queries were safe then we could use
-- the FORCED parameterization at the database-level.

-- However, as an alternative, you can also use this
-- with "plan guides" in SQL Server 2008.

DECLARE @SafeQuery nvarchar(max);
DECLARE @Parameters nvarchar(max);
EXEC sp_get_query_template 
    N'SELECT m.lastname, m.firstname, m.phone_no 
        FROM dbo.member AS m
        WHERE m.lastname = ''Chen''',
    @SafeQuery OUTPUT, 
    @Parameters OUTPUT;
select @SafeQuery, @Parameters
EXEC sp_create_plan_guide 
    N'Member: Query used for (view) range phone lookup', -- come up with a standard naming convention
    @SafeQuery, 
    N'TEMPLATE', 
    NULL, 
    @Parameters, 
    N'OPTION(PARAMETERIZATION FORCED)';
SELECT @SafeQuery, @Parameters
go

SELECT * FROM sys.plan_guides
go

sp_control_plan_guide N'enable',N'Guide_For_GetMemberInfo'
go


-- What about an Unsafe plan?
DECLARE @UnSafeQuery nvarchar(max);
DECLARE @Parameters nvarchar(max);
EXEC sp_get_query_template 
    N'SELECT m.lastname, m.firstname, m.phone_no 
        FROM dbo.member AS m
        WHERE m.lastname = ''Chen''',
    @UnSafeQuery OUTPUT, 
    @Parameters OUTPUT;
EXEC sp_create_plan_guide 
    N'Member: Query used for (view) range phone lookup', -- come up with a standard naming convention
    @UnSafeQuery, 
    N'TEMPLATE', 
    NULL, 
    @Parameters, 
    N'OPTION(PARAMETERIZATION SIMPLE)';
SELECT @UnSafeQuery, @Parameters
go