/*============================================================================
  File:     Filtered Statistics.sql

  Summary:  Setup and investigate Filtered Indexes

  Date:     October 2012

  SQL Server Version: SQL Server 2008+
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp & Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended as a supplement to the SQL Server 2008 Jumpstart or
  Metro training.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE [AdventureWorksDW2008_ModifiedSalesKey]
go

--------------------------------
-- Clean up
--------------------------------
sp_helpstats FactInternetSales
go

--DROP STATISTICS FactInternetSales.[SalesByCustomer_R11000-12000]
go

sp_helpindex FactInternetSales
go

--------------------------------
-- Analyze the data
--------------------------------
SELECT S.CustomerKey, COUNT(*) AS NumTotalSales
FROM FactInternetSales AS S
GROUP BY S.CustomerKey
ORDER BY NumTotalSales DESC
go

DBCC SHOW_STATISTICS(FactInternetSales, IX_FactInternetSales_CustomerKey)
--WITH HISTOGRAM
WITH STAT_HEADER JOIN DENSITY_VECTOR
-- Take table_cardinality/tuple_cardinality
-- select 30923776/18484
go

SELECT COUNT(*) from Dimcustomer -- 18,484

SELECT 30923776 * 5.410084E-05
SELECT 30923776 * 7.061445E-08
SELECT 30923776 * 3.233758E-08
go

-- NOTE: These statistics were updated with FULL SCAN
--		 They are as detailed as they could be.

--RANGE_HI_KEY	RANGE_ROWS	EQ_ROWS		DISTINCT_RANGE_ROWS	AVG_RANGE_ROWS
--11498			--			--			--					--
--11519			140288		18432		20					7014.4

-- So, what does it tell us for any value supplied 
-- BETWEEN 11498 and 11519...
-- Only the expected number of rows - which is 7014.4

-- Wait... what if they're NOT evenly distributed??

SELECT S.CustomerKey, COUNT(*) AS NumTotalSales
FROM FactInternetSales AS S
WHERE S.CustomerKey > 11498 and S.CustomerKey < 11519
GROUP BY S.CustomerKey
ORDER BY NumTotalSales DESC
go

-- In this range:
-- 11502 has the highest number of rows: 22016
-- 11512 has the lowest number of rows: 512

SET STATISTICS IO ON
SET STATISTICS TIME ON
go

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11502
go

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11512
go

------------------------------------------------------------------------------
-- Create a filtered statistics to help better define a range
------------------------------------------------------------------------------
CREATE STATISTICS [SalesByCustomer_R11000-12000] 
	ON FactInternetSales (CustomerKey)
WHERE CustomerKey >= 11000 
	AND CustomerKey <= 12000
	--WITH FULLSCAN
	go

DBCC SHOW_STATISTICS (FactInternetSales, [SalesByCustomer_R11000-12000])
WITH HISTOGRAM
go

sp_recompile FactInternetSales
go

-- Do the query plans change?

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11502
go

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11512
go