/*============================================================================
  File:     dm_exec_procedure_stats.sql

  Summary:  A few ways to query the procedure stats...

  Date:     October 2011

  SQL Server Version: 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- This is really similar to dm_exec_query_stats but at the procedure level
-- find procedures that are executed frequently and/or ones that have
-- high cumulative costs

-- Cool thing to add - database/object_name

SELECT db_name(database_id) AS 'Database Name'
    , OBJECT_NAME(object_id, database_id) AS 'Object Name'
    , * 
FROM sys.dm_exec_procedure_stats
ORDER BY 1, 2
go

SELECT db_name(database_id) AS 'Database Name'
    , OBJECT_NAME(object_id, database_id) AS 'Object Name'
    , execution_count
    , total_elapsed_time
    , * 
FROM sys.dm_exec_procedure_stats
ORDER BY 4 DESC
go

-- Query dm_exec_cached_plans for query plans

SELECT db_name(ps.database_id) AS 'Database Name'
    , OBJECT_NAME(ps.object_id, ps.database_id) AS 'Object Name'
    , ps.execution_count
    , ps.total_elapsed_time
    , ps.* 
    , cp.*
    , st.*
    , qp.*
FROM sys.dm_exec_procedure_stats AS ps
    JOIN sys.dm_exec_cached_plans AS cp
        ON ps.plan_handle = cp.plan_handle
    CROSS APPLY sys.dm_exec_sql_text (ps.sql_handle) AS st
    CROSS APPLY sys.dm_exec_query_plan (ps.plan_handle) AS qp
WHERE ps.database_id < 32767
ORDER BY 4 DESC
go

create proc MultiStatement
as
select * from sys.objects
select * from sys.all_columns
select * from sys.system_columns
go

exec MultiStatement
go

